@@ -8,33 +8,18 @@
 # ABSTRACT: Perl Globstar (double asterisk globbing) and utils
 
 package File::Globstar;
-$File::Globstar::VERSION = '0.2.1';
+
 use strict;
 
-use Locale::TextDomain qw(File-Globstar);
 use File::Glob qw(bsd_glob);
-use Scalar::Util 1.21 qw(reftype);
+use Scalar::Util qw(reftype);
 use File::Find;
 
 use base 'Exporter';
 use vars qw(@EXPORT_OK);
-@EXPORT_OK = qw(globstar fnmatchstar translatestar quotestar pnmatchstar);
-
-use constant RE_NONE => 0x0;
-use constant RE_NEGATED => 0x1;
-use constant RE_FULL_MATCH => 0x2;
-use constant RE_DIRECTORY => 0x4;
-
-# Remember what Scalar::Util::reftype() returns for a compiled regular
-# expression.  It should normally be 'REGEXP' but with Perl 5.10 (or
-# maybe older) this seems to be an empty string.  In this case, the
-# check in pnmatchstar() whether it received a compiled regex will be
-# rather weak ...
-my $test_re = qr/./;
-my $regex_type = reftype $test_re;
+@EXPORT_OK = qw(globstar fnmatchstar translatestar quotestar);
 
 sub _globstar;
-sub pnmatchstar;
 
 sub empty($) {
     my ($what) = @_;
@@ -231,21 +216,6 @@ sub _transpile_range($) {
 sub translatestar {
     my ($pattern, %options) = @_;
 
-    die __x("invalid pattern '{pattern}'\n", pattern => $pattern)
-        if $pattern =~ m{^/+$};
-
-    my $blessing = RE_NONE;
-
-    if ($options{pathMode}) {
-        $blessing |= RE_NEGATED if $pattern =~ s/^!//;
-        $blessing |= RE_DIRECTORY if $pattern =~ s{/$}{};
-        $blessing |= RE_FULL_MATCH if $pattern =~ m{/};
-        $pattern =~ s{^/}{};
-    }
-
-    # xgettext doesn't parse Perl code in regexes.
-    my $invalid_msg = __"invalid use of double asterisk";
-
     $pattern =~ s
                 {
                     (.*?)               # Anything, followed by ...
@@ -296,70 +266,34 @@ sub translatestar {
                         $translated .= _transpile_range $2;
                     } elsif (length $2) {
                         if ($2 =~ /\*\*/) {
-                            die $invalid_msg;
+                            die "invalid use of double asterisk";
                         }
                         die "should not happen: $2"; 
                     }
                     $translated;
                 }gsex;
 
-    my $re = $options{ignoreCase} ? qr/^$pattern$/i : qr/^$pattern$/;
-
-    bless $re, $blessing;
+    return $options{ignoreCase} ? qr/^$pattern$/i : qr/^$pattern$/;
 }
 
 sub fnmatchstar {
     my ($pattern, $string, %options) = @_;
 
     my $transpiled = eval { translatestar $pattern, %options };
-    return if $@;
+    if ($@) {
+        if ($options{ignoreCase}) {
+            lc $pattern eq lc $string or return;
+        } else {
+            $pattern eq $string or return;
+        }
+
+        return 1;
+    }
 
     $string =~ $transpiled or return;
 
     return 1;
 }
 
-sub pnmatchstar {
-    my ($pattern, $string, %options) = @_;
-
-    $options{isDirectory} = 1 if $string =~ s{/$}{};
-
-    my $full_path = $string;
-
-    # Check whether the regular expression is compiled.  
-    # (ref $pattern) may be false here because it can be 0.
-    my $reftype = reftype $pattern;
-    unless (defined $reftype && $regex_type eq $reftype) {
-        $pattern = eval { translatestar $pattern, %options, pathMode => 1 };
-        return if $@;
-    }
-
-    my $flags = ref $pattern;
-    $string =~ s{.*/}{} unless $flags & RE_FULL_MATCH;
-
-    my $match = $string =~ $pattern;
-    if ($flags & RE_DIRECTORY) {
-        undef $match if !$options{isDirectory};        
-    }
-
-    my $negated = $flags & RE_NEGATED;
-
-    if ($match) {
-        if ($negated) {
-            return;
-       } else {
-            return 1;
-        }
-    }
-
-    if ($full_path =~ s{/[^/]*$}{}) {
-        return pnmatchstar $pattern, $full_path, %options, isDirectory => 1;
-    }
-
-    return 1 if $negated;
-    
-    return;
-}
-
 1;
 
